try:
    import json
except ImportError:
    from django.utils import simplejson as json
    
from respite.serializers.base import Serializer
    
class JSONSerializer(Serializer):

    def serialize(self):
        data = super(JSONSerializer, self).serialize()

        return json.dumps(data, ensure_ascii=False)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

# A string describing the format that Respite will fall back to if
# none of the formats the client requested are supported by the view.
#
# Examples:
# DEFAULT_FORMAT = 'HyperText Markup Language'
# DEFAULT_FORMAT = 'HTML'
# DEFAULT_FORMAT = 'html
DEFAULT_FORMAT = getattr(settings, 'RESPITE_DEFAULT_FORMAT', None)



from functools import wraps

from respite.urls import routes

def override_supported_formats(formats):
    """
    Override the views class' supported formats for the decorated function.

    Arguments:
    formats -- A list of strings describing formats, e.g. ['html', 'json']
    """
    def decorator(function):
        @wraps(function)
        def wrapper(self, *args, **kwargs):
            self.supported_formats = formats
            return function(self, *args, **kwargs)
        return wrapper
    return decorator

def route(regex, method, name):
    """
    Route the decorated view.

    :param regex: A string describing a regular expression to which the request path will be matched.
    :param method: A string describing the HTTP method that this view accepts.
    :param name: A string describing the name of the URL pattern.
    
    ``regex`` may also be a lambda that accepts the parent resource's ``prefix`` argument and returns
    a string describing a regular expression to which the request path will be matched.
    
    ``name`` may also be a lambda that accepts the parent resource's ``views`` argument and returns
    a string describing the name of the URL pattern.
    """

    def decorator(function):
        function.route = routes.route(
            regex = regex,
            view = function.__name__,
            method = method,
            name = name
        )

        @wraps(function)
        def wrapper(self, *args, **kwargs):
            return function(self, *args, **kwargs)
        return wrapper

    return decorator



from xml.etree import ElementTree as ET

from respite.inflector import singularize
from respite.serializers.base import Serializer

class XMLSerializer(Serializer):

    def serialize(self):
        data = super(XMLSerializer, self).serialize()

        root = ET.Element('response')

        def serialize(key, value):

            def serialize_boolean(boolean):
                element = ET.Element(key)
                element.text = 'true' if boolean else 'false'

                return element

            def serialize_string(string):
                element = ET.Element(key)
                element.text = string

                return element

            def serialize_list(list):
                element = ET.Element(key)

                for item in list:
                    subelement = serialize(singularize(key), item)
                    element.append(subelement)

                return element

            def serialize_dictionary(dictionary):
                element = ET.Element(key)

                for subelement_key, subelement_value in dictionary.items():
                    subelements = serialize(subelement_key, subelement_value)
                    element.append(subelements)

                return element

            def serialize_integer(integer):
                element = ET.Element(key)
                element.text = '%s' % integer

                return element

            def serialize_none(none):
                element = ET.Element(key)
                element.text = ''

                return element

            if isinstance(value, bool):
                return serialize_boolean(value)

            if isinstance(value, basestring):
                return serialize_string(value)

            if isinstance(value, list):
                return serialize_list(value)

            if isinstance(value, dict):
                return serialize_dictionary(value)

            if isinstance(value, int):
                return serialize_integer(value)

            if value is None:
                return serialize_none(value)

            raise TypeError("Respite doesn't know how to serialize %s as XML" % value.__class__.__name__)

        for key, value in data.items():
            root.append(serialize(key, value))

        return '<?xml version="1.0" encoding="UTF-8"?>' + ET.tostring(root)



import re

def pluralize(word) :
        """Pluralize an English noun."""

        rules = [
            ['(?i)(quiz)$' , '\\1zes'],
            ['^(?i)(ox)$' , '\\1en'],
            ['(?i)([m|l])ouse$' , '\\1ice'],
            ['(?i)(matr|vert|ind)ix|ex$' , '\\1ices'],
            ['(?i)(x|ch|ss|sh)$' , '\\1es'],
            ['(?i)([^aeiouy]|qu)ies$' , '\\1y'],
            ['(?i)([^aeiouy]|qu)y$' , '\\1ies'],
            ['(?i)(hive)$' , '\\1s'],
            ['(?i)(?:([^f])fe|([lr])f)$' , '\\1\\2ves'],
            ['(?i)sis$' , 'ses'],
            ['(?i)([ti])um$' , '\\1a'],
            ['(?i)(buffal|tomat)o$' , '\\1oes'],
            ['(?i)(bu)s$' , '\\1ses'],
            ['(?i)(alias|status)' , '\\1es'],
            ['(?i)(octop|vir)us$' , '\\1i'],
            ['(?i)(ax|test)is$' , '\\1es'],
            ['(?i)s$' , 's'],
            ['(?i)$' , 's']
        ]

        uncountable_words = ['equipment', 'information', 'rice', 'money', 'species', 'series', 'fish', 'sheep']

        irregular_words = {
            'person' : 'people',
            'man' : 'men',
            'child' : 'children',
            'sex' : 'sexes',
            'move' : 'moves'
        }

        lower_cased_word = word.lower();

        for uncountable_word in uncountable_words:
            if lower_cased_word[-1*len(uncountable_word):] == uncountable_word :
                return word

        for irregular in irregular_words.keys():
            match = re.search('('+irregular+')$',word, re.IGNORECASE)
            if match:
                return re.sub('(?i)'+irregular+'$', match.expand('\\1')[0]+irregular_words[irregular][1:], word)

        for rule in range(len(rules)):
            match = re.search(rules[rule][0], word, re.IGNORECASE)
            if match :
                groups = match.groups()
                for k in range(0,len(groups)) :
                    if groups[k] == None :
                        rules[rule][1] = rules[rule][1].replace('\\'+str(k+1), '')

                return re.sub(rules[rule][0], rules[rule][1], word)

        return word

def singularize(word) :
        """Singularize an English noun."""

        rules = [
            ['(?i)(quiz)zes$' , '\\1'],
            ['(?i)(matr)ices$' , '\\1ix'],
            ['(?i)(vert|ind)ices$' , '\\1ex'],
            ['(?i)^(ox)en' , '\\1'],
            ['(?i)(alias|status)es$' , '\\1'],
            ['(?i)([octop|vir])i$' , '\\1us'],
            ['(?i)(cris|ax|test)es$' , '\\1is'],
            ['(?i)(shoe)s$' , '\\1'],
            ['(?i)(o)es$' , '\\1'],
            ['(?i)(bus)es$' , '\\1'],
            ['(?i)([m|l])ice$' , '\\1ouse'],
            ['(?i)(x|ch|ss|sh)es$' , '\\1'],
            ['(?i)(m)ovies$' , '\\1ovie'],
            ['(?i)(s)eries$' , '\\1eries'],
            ['(?i)([^aeiouy]|qu)ies$' , '\\1y'],
            ['(?i)([lr])ves$' , '\\1f'],
            ['(?i)(tive)s$' , '\\1'],
            ['(?i)(hive)s$' , '\\1'],
            ['(?i)([^f])ves$' , '\\1fe'],
            ['(?i)(^analy)ses$' , '\\1sis'],
            ['(?i)((a)naly|(b)a|(d)iagno|(p)arenthe|(p)rogno|(s)ynop|(t)he)ses$' , '\\1\\2sis'],
            ['(?i)([ti])a$' , '\\1um'],
            ['(?i)(n)ews$' , '\\1ews'],
            ['(?i)s$' , ''],
        ];

        uncountable_words = ['equipment', 'information', 'rice', 'money', 'species', 'series', 'fish', 'sheep','sms'];

        irregular_words = {
            'people' : 'person',
            'men' : 'man',
            'children' : 'child',
            'sexes' : 'sex',
            'moves' : 'move'
        }

        lower_cased_word = word.lower();

        for uncountable_word in uncountable_words:
            if lower_cased_word[-1*len(uncountable_word):] == uncountable_word :
                return word

        for irregular in irregular_words.keys():
            match = re.search('('+irregular+')$',word, re.IGNORECASE)
            if match:
                return re.sub('(?i)'+irregular+'$', match.expand('\\1')[0]+irregular_words[irregular][1:], word)

        for rule in range(len(rules)):
            match = re.search(rules[rule][0], word, re.IGNORECASE)
            if match :
                groups = match.groups()
                for k in range(0,len(groups)) :
                    if groups[k] == None :
                        rules[rule][1] = rules[rule][1].replace('\\'+str(k+1), '')

                return re.sub(rules[rule][0], rules[rule][1], word)

        return word

def cc2us(string):
    """Transform a CamelCase string to an underscore_case string."""
    return re.sub(r'(?<=[a-z])[A-Z]|(?<!^)[A-Z](?=[a-z])', r"_\g<0>", string).lower()

def mc2us(string):
    """Transform a mixedCase string to an underscore_case string."""
    return cc2us(string)

def us2mc(string):
    """Transform an underscore_case string to a mixedCase string"""
    return re.sub(r'_([a-z])', lambda m: (m.group(1).upper()), string)

def us2cc(string):
    """Transform an underscore_case string to a CamelCase string"""
    return us2mc(string)[0].upper()+us2mc(string)[1:]



from respite.inflector import pluralize, cc2us

def route(regex, view, method, name):
    """
    Route the given view.
    
    :param regex: A string describing a regular expression to which the request path will be matched.
    :param view: A string describing the name of the view to delegate the request to.
    :param method: A string describing the HTTP method that this view accepts.
    :param name: A string describing the name of the URL pattern.
    
    ``regex`` may also be a lambda that accepts the parent resource's ``prefix`` argument and returns
    a string describing a regular expression to which the request path will be matched.
    
    ``name`` may also be a lambda that accepts the parent resource's ``views`` argument and returns
    a string describing the name of the URL pattern.
    """
    return _Route(regex, view, method, name)

class _Route(object):

    def __init__(self, regex, view, method, name):
        self.regex = regex
        self.view = view
        self.method = method
        self.name = name



from django.shortcuts import render
from django.http import HttpResponse
from django.core.urlresolvers import reverse
from django.forms import CharField, HiddenInput
from django.forms.models import model_to_dict
from django.db.models import FieldDoesNotExist

from respite.utils import generate_form
from respite.inflector import pluralize, cc2us
from respite.views.views import Views

from respite.decorators import route

class Resource(object):
    """
    A collection of views that facilitate for common features.
    
    :attribute model: A reference to a model.
    :attribute form: A reference to a form, or ``None`` to generate one automatically.
    """
    
    model = None
    form = None

    @route(
        regex = lambda prefix: r'^%s(?:$|index(?:\.[a-zA-Z]+)?$)' % prefix,
        method = 'GET',
        name = lambda views: pluralize(cc2us(views.model.__name__))
    )
    def index(self, request):
        """Render a list of objects."""
        objects = self.model.objects.all()

        return self._render(
            request = request,
            template = 'index',
            context = {
                cc2us(pluralize(self.model.__name__)): objects,
            },
            status = 200
        )

    @route(
        regex = lambda prefix: r'^%s(?P<id>[0-9]+)(?:\.[a-zA-Z]+)?$' % prefix,
        method = 'GET',
        name = lambda views: cc2us(views.model.__name__)
    )
    def show(self, request, id):
        """Render a single object."""
        try:
            object = self.model.objects.get(id=id)
        except self.model.DoesNotExist:
            return self._render(
                request = request,
                template = '404',
                context = {
                    'error': 'The %s could not be found.' % self.model.__name__.lower()
                },
                status = 404,
                prefix_template_path = False
            )

        return self._render(
            request = request,
            template = 'show',
            context = {
                self.model.__name__.lower(): object
            },
            status = 200
        )

    @route(
        regex = lambda prefix: r'^%snew(?:\.[a-zA-Z]+)?$' % prefix,
        method = 'GET',
        name = lambda views: 'new_%s' % cc2us(views.model.__name__)
    )
    def new(self, request):
        """Render a form to create a new object."""
        form = (self.form or generate_form(self.model))()

        return self._render(
            request = request,
            template = 'new',
            context = {
                'form': form
            },
            status = 200
        )

    @route(
        regex = lambda prefix: r'^%s(?:$|index(?:\.[a-zA-Z]+)?$)' % prefix,
        method = 'POST',
        name = lambda views: pluralize(cc2us(views.model.__name__))
    )
    def create(self, request):
        """Create a new object."""    
        form = (self.form or generate_form(self.model))(request.POST)

        if form.is_valid():
            object = form.save()

            return self._render(
                request = request,
                template = 'show',
                context = {
                    self.model.__name__.lower(): object
                },
                status = 201
            )
        else:
            return self._render(
                request = request,
                template = 'new',
                context = {
                    'form': form
                },
                status = 400
            )

    @route(
        regex = lambda prefix: r'^%s(?P<id>[0-9]+)/edit(?:\.[a-zA-Z]+)?$' % prefix,
        method = 'GET',
        name = lambda views: 'edit_%s' % cc2us(views.model.__name__)
    )
    def edit(self, request, id):
        """Render a form to edit an object."""
        try:
            object = self.model.objects.get(id=id)
        except self.model.DoesNotExist:
            return self._render(
                request = request,
                template = '404',
                context = {
                    'error': 'The %s could not be found.' % self.model.__name__.lower()
                },
                status = 404,
                prefix_template_path = False
            )

        form = (self.form or generate_form(self.model))(instance=object)

        # Add "_method" field to override request method to PUT
        form.fields['_method'] = CharField(required=True, initial='PUT', widget=HiddenInput)

        return self._render(
            request = request,
            template = 'edit',
            context = {
                self.model.__name__.lower(): object,
                'form': form
            },
            status = 200
        )

    @route(
        regex = lambda prefix: r'^%s(?P<id>[0-9]+)(?:\.[a-zA-Z]+)?$' % prefix,
        method = 'PATCH',
        name = lambda views: cc2us(views.model.__name__)
    )
    def update(self, request, id):
        """Update an object."""
        try:
            object = self.model.objects.get(id=id)
        except self.model.DoesNotExist:
            return self._render(
                request = request,
                template = '404',
                context = {
                    'error': 'The %s could not be found.' % self.model.__name__.lower()
                },
                status = 404,
                prefix_template_path = False
            )

        fields = []
        for field in request.PATCH:

            try:
                self.model._meta.get_field_by_name(field)
            except FieldDoesNotExist:
                continue
            else:
                fields.append(field)

        Form = generate_form(
            model = self.model,
            form = self.form,
            fields = fields
        )

        form = Form(request.PATCH, instance=object)
        if form.is_valid():
            object = form.save()

            return self.show(request, id)
        else:
            return self._render(
                request = request,
                template = 'edit',
                context = {
                    'form': form
                },
                status = 400
            )

    @route(
        regex = lambda prefix: r'^%s(?P<id>[0-9]+)(?:\.[a-zA-Z]+)?$' % prefix,
        method = 'PUT',
        name = lambda views: cc2us(views.model.__name__)
    )
    def replace(self, request, id):
        """Replace an object."""
        try:
            object = self.model.objects.get(id=id)
        except self.model.DoesNotExist:
            return self._render(
                request = request,
                template = '404',
                context = {
                    'error': 'The %s could not be found.' % self.model.__name__.lower()
                },
                status = 404,
                prefix_template_path = False
            )

        form = (self.form or generate_form(self.model))(request.PUT, instance=object)

        if form.is_valid():
            object = form.save()

            return self.show(request, id)
        else:
            return self._render(
                request = request,
                template = 'edit',
                context = {
                    'form': form
                },
                status = 400
            )

    @route(
        regex = lambda prefix: r'^%s(?P<id>[0-9]+)(?:\.[a-zA-Z]+)?$' % prefix,
        method = 'DELETE',
        name = lambda views: cc2us(views.model.__name__)
    )
    def destroy(self, request, id):
        """Delete an object."""
        try:
            object = self.model.objects.get(id=id)
            object.delete()
        except self.model.DoesNotExist:
            return self._render(
                request = request,
                template = '404',
                context = {
                    'error': 'The %s could not be found.' % self.model.__name__.lower()
                },
                status = 404,
                prefix_template_path = False
            )

        return self._render(
            request = request,
            template = 'destroy',
            status = 200
        )

    routes = [
        index.route, show.route, new.route, create.route,
        edit.route, update.route, replace.route, destroy.route
    ]



try:
    from collections import OrderedDict
except ImportError:
    from ..lib.ordereddict import OrderedDict

import django.db.models
import django.forms
import datetime

class Serializer(object):
    """Base class for serializers."""

    def __init__(self, source):
        self.source = source

    def serialize(self):
        """
        Serialize the given object into into simple
        data types (e.g. lists, dictionaries, strings).
        """

        def serialize(anything):
            
            def serialize_dictionary(dictionary):
                """Dictionaries are serialized recursively."""
                data = OrderedDict()

                # Serialize each of the dictionary's keys
                for key, value in dictionary.items():
                    data.update({ key: serialize(value) })

                return data
                
            def serialize_list(list):
                """Lists are serialized recursively."""
                data = []
                
                # Serialize each of the list's items
                for item in list:
                    data.append(serialize(item))
                    
                return data

            def serialize_queryset(queryset):
                """Querysets are serialized as lists of models."""
                data = []

                # Serialize queryset as a list of models
                for model in queryset:
                    data.append(serialize_model(model))

                return data

            def serialize_manager(manager):
                """Managers are serialized as list of models."""
                data = []

                for model in manager.all():
                    data.append(serialize_model(model))

                return data

            def serialize_model(model):
                """
                Models are serialized by calling their 'serialize' method.

                Models that don't define a 'serialize' method are
                serialized as a dictionary of fields.

                Example:

                    {
                        'id': 1,
                        'title': 'Mmmm pie',
                        'content: 'Pie is good!'
                    }

                """

                if hasattr(model, 'serialize'):
                    return serialize(model.serialize())
                else:
                    data = OrderedDict()
                    for field in model._meta.fields + model._meta.many_to_many:
                        data.update({
                            field.name: serialize(getattr(model, field.name ))
                        })

                    return data

            def serialize_form(form):
                """
                Forms are serialized as a dictionary of fields and errors (if any).

                Example:

                    {
                        'fields': ['title', 'content'],
                        'errors': {
                            'content': 'Must describe pie.'
                        }
                    }

                """
                data = OrderedDict()

                # Serialize form fields as a list of strings
                data['fields'] = []
                for field in form.fields:
                    data['fields'].append(field)

                # Serialize form errors as a dictionary with keys 'field' and 'error'
                if form.errors:
                    data['errors'] = []
                    for field in form:
                        data['errors'].append(
                            {
                                'field': field.name,
                                'error': field.errors.as_text()
                            }
                        )

                return data

            def serialize_date(datetime):
                """Dates are serialized as ISO 8601-compatible strings."""
                return datetime.isoformat()
            
            def serialize_field_file(field_file):
                """Filefields are serialized as strings describing their URL."""
                try:
                    return field_file.url
                except ValueError:
                    return None

            def serialize_image_field_file(image_field_file):
                """Imagefields are serialized as strings describing their URL."""
                try:
                    return image_field_file.url
                except ValueError:
                    return None

            if isinstance(anything, dict):
                return serialize_dictionary(anything)

            if isinstance(anything, list):
                return serialize_list(anything)

            if isinstance(anything, django.db.models.query.QuerySet):
                return serialize_queryset(anything)

            if isinstance(anything, django.db.models.Model):
                return serialize_model(anything)

            if isinstance(anything, (django.forms.Form, django.forms.ModelForm)):
                return serialize_form(anything)

            if isinstance(anything, (str, unicode)):
                return anything

            if isinstance(anything, (int, float, long)):
                return anything

            if isinstance(anything, (datetime.date, datetime.datetime)):
                return serialize_date(anything)

            if isinstance(anything, django.db.models.manager.Manager):
                return serialize_manager(anything)

            if type(anything) is django.db.models.fields.files.FieldFile:
                return serialize_field_file(anything)

            if type(anything) is django.db.models.fields.files.ImageFieldFile:
                return serialize_image_field_file(anything)

            if anything is None:
                return None

            if hasattr(anything, 'serialize'):
                return serialize(anything.serialize())

            raise TypeError("Respite doesn't know how to serialize %s" % anything.__class__.__name__)

        return serialize(self.source)



import re

from urllib import urlencode

from django.http import QueryDict
from django.utils import simplejson as json

from respite.utils import parse_content_type

class HttpMethodOverrideMiddleware:
    """
    Facilitate for overriding the HTTP method with the X-HTTP-Method-Override
    header or a '_method' HTTP POST parameter.
    """

    def process_request(self, request):
        if 'HTTP_X_HTTP_METHOD_OVERRIDE' in request.META \
        or '_method' in request.POST:
            request.method = (
                request.META.get('HTTP_X_HTTP_METHOD_OVERRIDE') or
                request.POST.get('_method')
            ).upper()

        if '_method' in request.POST:
            request._raw_post_data = re.sub(r'_method=(PUT|PATCH|DELETE)&?', '', request.raw_post_data)

class HttpPutMiddleware:
    """
    Facilitate for HTTP PUT in the same way Django facilitates for HTTP GET
    and HTTP POST; populate a QueryDict instance with the request body in request.PUT.
    """

    def process_request(self, request):
        if request.method == 'PUT':
            request.PUT = QueryDict(request.raw_post_data)

class HttpPatchMiddleware:
    """
    Facilitate for HTTP PATCH in the same way Django facilitates for HTTP GET
    and HTTP POST; populate a QueryDict instance with the request body in request.PATCH.
    """

    def process_request(self, request):
        if request.method == 'PATCH':
            request.PATCH = QueryDict(request.raw_post_data)

class JsonMiddleware:
    """
    Parse JSON in POST, PUT and PATCH requests.
    """

    def process_request(self, request):
        content_type = request.META.get('CONTENT_TYPE')

        if content_type and parse_content_type(content_type)[0] == 'application/json':
            data = json.loads(request.raw_post_data)

            if request.method in ['POST', 'PUT', 'PATCH']:
                setattr(request, request.method, QueryDict(urlencode(data)))



from respite.serializers.jsonserializer import JSONSerializer
from respite.serializers.xmlserializer import XMLSerializer
from respite import formats

SERIALIZERS = {
    formats.find('JavaScript Object Notation'): JSONSerializer,
    formats.find('Extensible Markup Language'): XMLSerializer
}

def find(format):
    """
    Find and return a serializer for the given format.

    Arguments:
    format -- A Format instance.
    """
    try:
        serializer = SERIALIZERS[format]
    except KeyError:
        raise UnknownSerializer('No serializer found for %s' % format.acronym)

    return serializer

class UnknownSerializer(Exception):
    pass



from copy import deepcopy

from django.conf.urls.defaults import *
from django.http import HttpResponse

from respite.inflector import pluralize, cc2us
from respite.urls import routes

def resource(views, routes, prefix=''):
    """
    Route a collection of views.

    :param views: A reference to the class that defines the views.
    :param routes: A list of routes.
    :param prefix: A string to prefix the routes by (e.g. 'posts/'). Defaults to an empty string.
    """
    routes = deepcopy(routes)

    def dispatch(request, GET=False, POST=False, PUT=False, DELETE=False, PATCH=False, **kwargs):
        """
        Dispatch the request according to the request method and the string contained in
        the corresponding keyword argument.

        For example, if the request method is HTTP GET and the 'GET' argument to this function is
        set to 'index', the 'index' function of the views class will be invoked and returned.

        Arguments:
        :param request: A django.http.HttpRequest object.
        :param GET: A string describing the function to delegate the request to on HTTP GET.
        :param POST: A string describing the function to delegate the request to on HTTP POST.
        :param PUT: A string describing the function to delegate the request to on HTTP PUT.
        :param DELETE: A string describing the function to delegate the request to on HTTP DELETE.
        :param PATCH: A string describing the function to delegate the request to on HTTP PATCH.
        """

        # Return HTTP 405 Method Not Allowed if the request method isn't routed
        if request.method == 'GET' and not GET \
        or request.method == 'POST' and not POST \
        or request.method == 'PUT' and not PUT \
        or request.method == 'DELETE' and not DELETE \
        or request.method == 'PATCH' and not PATCH \
        or request.method not in ['GET', 'POST', 'PUT', 'DELETE', 'PATCH', 'OPTIONS', 'HEAD']:
            allowed_methods = []

            if GET:
                allowed_methods.append('GET')
            if POST:
                allowed_methods.append('POST')
            if PUT:
                allowed_methods.append('PUT')
            if DELETE:
                allowed_methods.append('DELETE')

            response = HttpResponse()
            response.status_code = 405
            response['Allow'] = ', '.join(allowed_methods)
            return response

        # Dispatch the request
        if request.method in ['GET', 'HEAD']:
            return getattr(views(), GET)(request, **kwargs)
        if request.method == 'POST':
            return getattr(views(), POST)(request, **kwargs)
        if request.method == 'PUT':
            return getattr(views(), PUT)(request, **kwargs)
        if request.method == 'DELETE':
            return getattr(views(), DELETE)(request, **kwargs)
        if request.method == 'PATCH':
            return getattr(views(), PATCH)(request, **kwargs)
        if request.method == 'OPTIONS':
            map = {}

            if GET:
                map['GET'] = getattr(views(), GET)
            if POST:
                map['POST'] = getattr(views(), POST)
            if PUT:
                map['PUT'] = getattr(views(), PUT)
            if DELETE:
                map['DELETE'] = getattr(views(), DELETE)

            return views().options(request, map, **kwargs)

    def urlify(routes):
        """
        Transform routes into urlpatterns.

        Arguments:
        :param routes: A list of routes.
        """
        urls = []

        # Route regular expressions and names may be lambdas; expand them.
        for i, route in enumerate(routes):
            if callable(route.regex):
                routes[i].regex = route.regex(prefix)
            else:
                routes[i].regex = '^%s' % prefix + (route.regex[1:] if route.regex[0] == '^' else route.regex)

            if callable(route.name):
                routes[i].name = route.name(views)

        for route in list(routes):

            # Collect this route and its siblings (i.e. routes that share
            # same regular expression) in a dictionary of keys that describe
            # HTTP methods and values that describe the corresponding
            # view function.
            #
            # Example:
            #
            # {
            #   'GET': 'index',
            #   'POST': 'create'
            # }
            kwargs = {}
            for sibling in list(routes):
                if sibling.regex == route.regex:
                    kwargs[sibling.method] = sibling.view
                    routes.remove(sibling)

            urls.append(
                url(
                    regex = route.regex,
                    view = dispatch,
                    kwargs = kwargs,
                    name = route.name
                )
            )

        return urls

    return urlify(routes)



from django import forms

def generate_form(model, form=None, fields=False, exclude=False):
    """
    Generate a form from a model.

    :param model: A Django model.
    :param form: A Django form.
    :param fields: A list of fields to include in this form.
    :param exclude: A list of fields to exclude in this form.
    """
    _model, _fields, _exclude = model, fields, exclude

    class Form(form or forms.ModelForm):
        class Meta:
            model = _model

            if _fields is not False:
                fields = _fields

            if _exclude is not False:
                exclude = _exclude

    return Form

def parse_content_type(content_type):
    """
    Return a tuple of content type and charset.

    :param content_type: A string describing a content type.
    """
    if ';' in content_type:
        return content_type.split(';')
    else:
        return content_type, 'ISO-8859-1'

def parse_http_accept_header(header):
    """
    Return a list of content types listed in the HTTP Accept header
    ordered by quality.

    :param header: A string describing the contents of the HTTP Accept header.
    """
    components = [item.strip() for item in header.split(',')]

    l = []
    for component in components:
        if ';' in component:
            subcomponents = [item.strip() for item in component.split(';')]
            l.append(
                (
                    subcomponents[0], # eg. 'text/html'
                    subcomponents[1][2:] # eg. 'q=0.9'
                )
            )
        else:
            l.append((component, '1'))

    l.sort(
        key = lambda i: i[1],
        reverse = True
    )

    content_types = []
    for i in l:
        content_types.append(i[0])

    return content_types



from setuptools import setup

setup(
    name = 'django-respite',
    version = '1.0.0',
    description = "Respite conforms Django to Representational State Transfer (REST)",
    author = "Johannes Gorset",
    author_email = "jgorset@gmail.com",
    url = "http://github.com/jgorset/respite",
    packages = ['respite', 'respite.lib', 'respite.serializers', 'respite.urls', 'respite.views']
)



from django.shortcuts import render
from django.http import HttpResponse
from django.template import TemplateDoesNotExist
from django.conf import settings

from respite.settings import DEFAULT_FORMAT
from respite.utils import parse_http_accept_header
from respite import serializers
from respite import formats

class Views(object):
    """
    Base class for views.
    
    :attribute template_path: A string describing a path to prefix templates with, or ``''`` by default.
    :attribute supported_formats: A list of strings describing formats supported by these views, or ``['html']`` by default.
    """
    template_path = ''
    supported_formats = ['html']

    def options(self, request, map, *args, **kwargs):
        """List communication options."""
        options = {}
        for method, function in map.items():
            options[method] = function.__doc__

        return self._render(
            request = request,
            template = None,
            context = {
                'options': options
            },
            status = 200,
            headers = {
                'Allow': ', '.join(options.keys())
            }
        )

    def _get_format(self, request):
        """
        Determine and return a 'formats.Format' instance describing the most desired response format
        that is supported by these views.

        :param request: A django.http.HttpRequest instance.

        Formats specified by extension (e.g. '/articles/index.html') take precedence over formats
        given in the HTTP Accept header, even if it's a format that isn't known by Respite.

        If the request doesn't specify a format by extension (e.g. '/articles/' or '/articles/new')
        and none of the formats in the HTTP Accept header are supported, Respite will fall back
        on the format given in DEFAULT_FORMAT.
        """

        # Derive a list of 'formats.Format' instances from the list of formats these views support.
        supported_formats = [formats.find(format) for format in self.supported_formats]

        # Determine format by extension...
        if '.' in request.path:
            extension = request.path.split('.')[-1]

            try:
                format = formats.find_by_extension(extension)
            except formats.UnknownFormat:
                return None

            return format if format in supported_formats else None

        # Determine format by HTTP Accept header...
        if 'HTTP_ACCEPT' in request.META:

            # Parse the HTTP Accept header, returning a list of accepted content types sorted by quality
            for accepted_content_type in parse_http_accept_header(request.META['HTTP_ACCEPT']):

                # Default to the view's preferred format for the wilcard content type.
                if accepted_content_type == '*/*':
                    return supported_formats[0]

                try:
                    format = formats.find_by_content_type(accepted_content_type)
                except formats.UnknownFormat:
                    continue

                if format in supported_formats:
                    return format
                else:
                    continue

        # If none of the formats given in the HTTP 'accept' header are supported by these views,
        # or no HTTP 'accept' header was given at all, default to the format given in
        # DEFAULT_FORMAT if that's supported.
        if DEFAULT_FORMAT:
            default_format = formats.find(DEFAULT_FORMAT)
            if default_format in supported_formats:
                return default_format

    def _render(self, request, template=None, status=200, context={}, headers={}, prefix_template_path=True):
        """
        Render a HTTP response.
        
        :param request: A django.http.HttpRequest instance.
        :param template: A string describing the path to a template.
        :param status: An integer describing the HTTP status code to respond with.
        :param context: A dictionary describing variables to populate the template with.
        :param headers: A dictionary describing HTTP headers.
        :param prefix_template_path: A boolean describing whether to prefix the template with the view's template path.
        
        Please note that ``template`` must not specify an extension, as one will be appended
        according to the request format. For example, a value of ``blog/posts/index``
        would populate ``blog/posts/index.html`` for requests that query the resource's
        HTML representation.
        
        If no template that matches the request format exists at the given location, or if ``template`` is ``None``,
        Respite will attempt to serialize the template context automatically. You can change the way your models
        are serialized by defining ``serialize`` methods that return a dictionary::
        
            class NuclearMissile(models.Model):
                serial_number = models.IntegerField()
                is_armed = models.BooleanField()
                launch_code = models.IntegerField()
                
                def serialize(self):
                    return {
                        'serial_number': self.serial_number,
                        'is_armed': self.is_armed
                    }
        
        If the request format is not supported by the view (as determined by the ``supported_formats``
        property or a specific view's ``override_supported_formats`` decorator), this function will
        yield HTTP 406 Not Acceptable.
        """

        format = self._get_format(request)

        # Render 406 Not Acceptable if the requested format isn't supported.
        if not format:
            return HttpResponse(status=406)

        if template:

            if prefix_template_path:
                template_path = '%s.%s' % (self.template_path + template, format.extension)
            else:
                template_path = '%s.%s' % (template, format.extension)

            try:
                response = render(
                    request = request,
                    template_name = template_path,
                    dictionary = context,
                    status = status,
                    content_type = '%s; charset=%s' % (format.content_type, settings.DEFAULT_CHARSET)
                )
            except TemplateDoesNotExist:
                response = HttpResponse(
                    content = serializers.find(format)(context).serialize(),
                    content_type = '%s; charset=%s' % (format.content_type, settings.DEFAULT_CHARSET),
                    status = status
                )
        else:
            response = HttpResponse(
                content = serializers.find(format)(context).serialize(),
                content_type = '%s; charset=%s' % (format.content_type, settings.DEFAULT_CHARSET),
                status = status
            )

        for header, value in headers.items():
            response[header] = value

        return response



class Format(object):
    """
    A format represents a file format.
    """

    def __init__(self, name, acronym, extensions, content_types):
        """
        Initialize a new format.

        :param name: A string describing the name of the format (e.g. 'HyperText Markup Language').
        :param acronym: A string describing the acronym of the format (e.g. 'HTML').
        :param extensions: A list of strings describing the extensions of the format (e.g. 'html').
        :param content_types: A list of strings describing the internet media type* of the format (e.g. 'text/html').

        * http://www.iana.org/assignments/media-types/index.html
        """
        self.name = name
        self.acronym = acronym
        self.extensions = extensions
        self.content_types = content_types

    @property
    def extension(self):
        """Return the desired extension for this format."""
        return self.extensions[0]

    @property
    def content_type(self):
        """Return the desired content type for this format."""
        return self.content_types[0]

    def __str__(self):
        return self.name

FORMATS = [
    Format('HyperText Markup Language', 'HTML', ['html'], ['text/html']),
    Format('Extensible Markup Language', 'XML', ['xml'], ['application/xml', 'text/xml']),
    Format('JavaScript Object Notation', 'JSON', ['json'], ['application/json']),
]

def find(identifier):
    """
    Find and return a format by name, acronym or extension.

    :param identifier: A string describing the format.
    """
    for format in FORMATS:
        if identifier in [format.name, format.acronym, format.extension]:
            return format

    raise UnknownFormat('No format found with name, acronym or extension "%s"' % identifier)

def find_by_name(name):
    """
    Find and return a format by name.

    :param name: A string describing the name of the format.
    """
    for format in FORMATS:
        if name == format.name:
            return format

    raise UnknownFormat('No format found with name "%s"' % name)

def find_by_extension(extension):
    """
    Find and return a format by extension.

    :param extension: A string describing the extension of the format.
    """
    for format in FORMATS:
        if extension in format.extensions:
            return format

    raise UnknownFormat('No format found with extension "%s"' % extension)

def find_by_content_type(content_type):
    """
    Find and return a format by content type.

    :param content_type: A string describing the internet media type of the format.
    """
    for format in FORMATS:
        if content_type in format.content_types:
            return format

    raise UnknownFormat('No format found with content type "%s"' % content_type)

class UnknownFormat(Exception):
    pass
